/**********************************************************************
 *                                                                    *
 *  cfKolabAdmin - QT application for simple user administration of   *
 *    a Kolab1 server.                                                *
 *                                                                    *
 *  Copyright (c) 2004  Code Fusion cc                                *
 *  Writen by                                                         *
 *    Stephan Buys  <s.buys@codefusion.co.za>                         *
 *    Stuart Bing  <s.binge@codefusion.co.za>                        *
 *                                                                    *
 *  This  program is free  software; you can redistribute  it and/or  *
 *  modify it  under the terms of the GNU  General Public License as  *
 *  published by the  Free Software Foundation; either version 2, or  *
 *  (at your option) any later version.                               *
 *                                                                    *
 *  This program is  distributed in the hope that it will be useful,  *
 *  but WITHOUT  ANY WARRANTY; without even the  implied warranty of  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  *
 *  General Public License for more details.                          *
 *                                                                    *
 *  You can view the  GNU General Public License, online, at the GNU  *
 *  Project's homepage; see <http://www.gnu.org/licenses/gpl.html>.   *
 *                                                                    *
 **********************************************************************/

#include "LdapAttributes.h"

#include <string>
#include <map>
#include <utility>

typedef std::map<std::string, std::string>      AttrDescMap;
typedef AttrDescMap::iterator                   AttrDescIter;

AttrDescMap attr_desc;

void setupAttrDesc()
{
    // Set up our attribute description map
    attr_desc.insert(std::make_pair("mail",            "Email Address"));
    attr_desc.insert(std::make_pair("sn",              "Surname"));
    attr_desc.insert(std::make_pair("firstName",       "First Name"));
    attr_desc.insert(std::make_pair("cn",              "Full Name"));
    attr_desc.insert(std::make_pair("alias",           "Email Alias"));
    attr_desc.insert(std::make_pair("userquota",       "Mailbox Quota (KB)"));
    attr_desc.insert(std::make_pair("deleteflag",      "Awaiting Removal"));
}

QString getAttrDesc(const char * attr)
{
    AttrDescIter i;
    return ((i = attr_desc.find(attr)) == attr_desc.end() ?
        QString(attr) : QString(i->second.c_str()));
}

void initLDAPAttributes(LdapAttrVec & attrs)
{
    attrs.push_back(0);
}

void addLDAPAttributes(LdapAttrVec & attrs, const char * name, const QString & value, int mode)
{
    QStringVec svec;
    svec.push_back(value);
    addLDAPAttributes(attrs, name, svec, mode);
}

void addLDAPAttributes(LdapAttrVec & attrs, const char * name, const QStringVec & values, int mode)
{
    LdapAttrIter attr = attrs.insert(attrs.begin(), new ldapmod);

    (*attr)->mod_op = mode;

    (*attr)->mod_type = new char [strlen(name) + 1];
    strcpy((*attr)->mod_type, name);
    (*attr)->mod_type[strlen(name)] = 0;

    (*attr)->mod_values = new char * [values.size() + 1];
    (*attr)->mod_values[values.size()] = 0;

    int pos = 0;
    for (QStringConstIter i = values.begin(); i != values.end(); ++i, ++pos) {
        (*attr)->mod_values[pos] = new char [i->length() + 1];
        strcpy((*attr)->mod_values[pos], i->ascii());
        (*attr)->mod_values[pos][i->length()] = 0;
    }
}

void clearLDAPAttributes(LdapAttrVec & attrs)
{
    ldap_mods_free(&attrs[0], 0);
    attrs.clear();
}
