/**********************************************************************
 *                                                                    *
 *  cfKolabAdmin - QT application for simple user administration of   *
 *    a Kolab1 server.                                                *
 *                                                                    *
 *  Copyright (c) 2004  Code Fusion cc                                *
 *  Writen by                                                         *
 *    Stephan Buys  <s.buys@codefusion.co.za>                         *
 *    Stuart Bing  <s.binge@codefusion.co.za>                        *
 *                                                                    *
 *  This  program is free  software; you can redistribute  it and/or  *
 *  modify it  under the terms of the GNU  General Public License as  *
 *  published by the  Free Software Foundation; either version 2, or  *
 *  (at your option) any later version.                               *
 *                                                                    *
 *  This program is  distributed in the hope that it will be useful,  *
 *  but WITHOUT  ANY WARRANTY; without even the  implied warranty of  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  *
 *  General Public License for more details.                          *
 *                                                                    *
 *  You can view the  GNU General Public License, online, at the GNU  *
 *  Project's homepage; see <http://www.gnu.org/licenses/gpl.html>.   *
 *                                                                    *
 **********************************************************************/

#ifndef HEADER_LDAPLISTVIEWITEM
#define HEADER_LDAPLISTVIEWITEM

#include <map>

#include <qobject.h>
#include <qstring.h>
#include <qdatetime.h>
#include <qptrlist.h>
#include <qlistview.h>
#include <ldap.h>

// Specialisation of QListViewItem to handle generic LDAP objects as list view items
// Note: This is an abstract class, and must never be instantiated.
class LdapListViewItem : public QListViewItem
{
    typedef std::multimap<QString, char **> AttrValueMap;
    typedef AttrValueMap::iterator          AttrValueIter;

protected:
    bool            has_loaded_attrs;
    AttrValueMap    attr_value_map;

    bool            awaiting_removal;

    LDAP          * ldap_connection;
    QString         entry_dn;

    QString loadAttrs(char ** attrs, bool * dne = 0);
    void clearAttrs();

public:
    LdapListViewItem(QListView * parent, LDAP * ldap_conn, LDAPMessage * ldap_entry);
    virtual ~LdapListViewItem() { clearAttrs(); }

    virtual void paintCell(QPainter * p, const QColorGroup & cg,
                           int column, int width, int alignment);
    virtual void paintFocus(QPainter * p, const QColorGroup & cg,
                            const QRect & r);

    QString reload(bool * dne = 0);
    QString remove();
    QString nuke();

    void populateListView(QListView * list_view);
    virtual QString getAttrs(bool * dne = 0) = 0;

    QString getAttrValue(const char * attr);
    QString getDN() const { return entry_dn; }
    bool awaitingRemoval() const { return awaiting_removal; }
};

#endif
