/**********************************************************************
 *                                                                    *
 *  cfKolabAdmin - QT application for simple user administration of   *
 *    a Kolab1 server.                                                *
 *                                                                    *
 *  Copyright (c) 2004  Code Fusion cc                                *
 *  Writen by                                                         *
 *    Stephan Buys  <s.buys@codefusion.co.za>                         *
 *    Stuart Bing  <s.binge@codefusion.co.za>                        *
 *                                                                    *
 *  This  program is free  software; you can redistribute  it and/or  *
 *  modify it  under the terms of the GNU  General Public License as  *
 *  published by the  Free Software Foundation; either version 2, or  *
 *  (at your option) any later version.                               *
 *                                                                    *
 *  This program is  distributed in the hope that it will be useful,  *
 *  but WITHOUT  ANY WARRANTY; without even the  implied warranty of  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  *
 *  General Public License for more details.                          *
 *                                                                    *
 *  You can view the  GNU General Public License, online, at the GNU  *
 *  Project's homepage; see <http://www.gnu.org/licenses/gpl.html>.   *
 *                                                                    *
 **********************************************************************/

#include "UserListViewItem.h"
#include "LdapAttributes.h"

char * ATTR_USER_LIST[] = {"mail", "deleteflag", 0};

char * ATTR_USER_FULL[] = {
    "mail",
    "sn",
    "firstName",
    "cn",
    "alias",
    "userquota",
    "deleteflag",
    0
};

UserListViewItem::UserListViewItem(QListView * parent, LDAP * ldap_conn, LDAPMessage * ldap_entry)
  : LdapListViewItem(parent, ldap_conn, ldap_entry)
{
    char ** values = ldap_get_values(ldap_connection, ldap_entry, "mail");
    setText(0, *values);
    ldap_value_free(values);
}

QString UserListViewItem::getAttrs(bool * dne)
{
    return loadAttrs(ATTR_USER_FULL, dne);
}

QString UserListViewItem::changePassword(const QString & password)
{
    // Change the users' password
    LdapAttrVec attrs;
    initLDAPAttributes(attrs);
    addLDAPAttributes(attrs, "userPassword", password, LDAP_MOD_REPLACE);

    int result = ldap_modify_s(ldap_connection, entry_dn.ascii(), &attrs[0]);

    clearLDAPAttributes(attrs);

    if (result == LDAP_SUCCESS) return QString::null;

    return ldap_err2string(result);
}
