/**********************************************************************
 *                                                                    *
 *  cfKolabAdmin - QT application for simple user administration of   *
 *    a Kolab1 server.                                                *
 *                                                                    *
 *  Copyright (c) 2004  Code Fusion cc                                *
 *  Writen by                                                         *
 *    Stephan Buys  <s.buys@codefusion.co.za>                         *
 *    Stuart Bing  <s.binge@codefusion.co.za>                        *
 *                                                                    *
 *  This  program is free  software; you can redistribute  it and/or  *
 *  modify it  under the terms of the GNU  General Public License as  *
 *  published by the  Free Software Foundation; either version 2, or  *
 *  (at your option) any later version.                               *
 *                                                                    *
 *  This program is  distributed in the hope that it will be useful,  *
 *  but WITHOUT  ANY WARRANTY; without even the  implied warranty of  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  *
 *  General Public License for more details.                          *
 *                                                                    *
 *  You can view the  GNU General Public License, online, at the GNU  *
 *  Project's homepage; see <http://www.gnu.org/licenses/gpl.html>.   *
 *                                                                    *
 **********************************************************************/

void dlgConnect::init()
{
    // Load any saved connection settings

    QDomDocument settings("settings");
    QFile settings_file("cfKolabAdmin.xml");
    if (!settings_file.open(IO_ReadOnly))
        return;

    if (!settings.setContent(&settings_file)) {
        settings_file.close();
        return;
    }

    settings_file.close();

    QDomElement docElem = settings.documentElement();

    QDomNode n = docElem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement();
        if (!e.isNull() && e.tagName() == "ldap_connection") {
            serverEdit->setText(e.attribute("server"));
            portEdit->setText(e.attribute("port"));
            baseDnEdit->setText(e.attribute("base_dn"));
            bindDnEdit->setText(e.attribute("bind_dn"));
            bindPwEdit->setText(e.attribute("bind_pw"));
            chkAppendBaseDN->setChecked(e.attribute("append_base_dn") == "true");
        }
        n = n.nextSibling();
    }
}

void dlgConnect::destroy()
{
    // Save out the current connection settings to a file

    QDomDocument document("settings");

    QDomElement settings = document.createElement("settings");
    QDomElement ldap_connection = document.createElement("ldap_connection");

    ldap_connection.setAttribute("server", getServer());
    ldap_connection.setAttribute("port", portEdit->text());
    ldap_connection.setAttribute("base_dn", getBaseDn());
    ldap_connection.setAttribute("bind_dn", getBindDn());
    ldap_connection.setAttribute("bind_pw", getBindPw());
    ldap_connection.setAttribute("append_base_dn", appendBaseDN() ? "true" : "false");

    document.appendChild(settings);
    settings.appendChild(ldap_connection);

    QFile settings_file("cfKolabAdmin.xml");
    if (!settings_file.open(IO_WriteOnly))
        return;

    QTextStream settings_stream(&settings_file);

    settings_stream << document.toString();
}

QString dlgConnect::getServer()
    { return serverEdit->text(); }

int dlgConnect::getPort()
    { return portEdit->text().toInt(); }

QString dlgConnect::getBindDn()
    { return bindDnEdit->text(); }

QString dlgConnect::getBindPw()
    { return bindPwEdit->text(); }

QString dlgConnect::getBaseDn()
    { return baseDnEdit->text(); }

bool dlgConnect::appendBaseDN()
    { return chkAppendBaseDN->isChecked(); }
