/**********************************************************************
 *                                                                    *
 *  cfKolabAdmin - QT application for simple user administration of   *
 *    a Kolab1 server.                                                *
 *                                                                    *
 *  Copyright (c) 2004  Code Fusion cc                                *
 *  Writen by                                                         *
 *    Stephan Buys  <s.buys@codefusion.co.za>                         *
 *    Stuart Bing  <s.binge@codefusion.co.za>                        *
 *                                                                    *
 *  This  program is free  software; you can redistribute  it and/or  *
 *  modify it  under the terms of the GNU  General Public License as  *
 *  published by the  Free Software Foundation; either version 2, or  *
 *  (at your option) any later version.                               *
 *                                                                    *
 *  This program is  distributed in the hope that it will be useful,  *
 *  but WITHOUT  ANY WARRANTY; without even the  implied warranty of  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  *
 *  General Public License for more details.                          *
 *                                                                    *
 *  You can view the  GNU General Public License, online, at the GNU  *
 *  Project's homepage; see <http://www.gnu.org/licenses/gpl.html>.   *
 *                                                                    *
 **********************************************************************/

// All the #ifdef WIN32 blocks are hacks to get the program to compile
// with QT/Win32 Free <http://kde-cygwin.sf.net/qt3-win32>, and to try
// and (somewhat) fix the redraw bugs that it also has (bugs 985725 and
// 985727 at <http://sourceforge.net/tracker/?group_id=27249&atid=389712>

void frmMain::init()
{
    ldap_connection = 0;
    ldap_protocol = LDAP_VERSION3;
    ldap_connected = false;
}

void frmMain::destroy()
{
    if (ldap_connection) {
        ldap_unbind_s(ldap_connection);
        ldap_connection = 0;
    }
}

void frmMain::slotConnect()
{
    // Set up our logging variables
    QString log_text;
    QTextOStream log(&log_text);

    dlgConnect dialog_connect(this);
    if (dialog_connect.exec() == QDialog::Rejected) {
        statusBar()->message("Cancelled");
        return;
    }

    ldap_base_dn = dialog_connect.getBaseDn();

    // Disconnect if we're already connected
    if (ldap_connection) {
        ldap_unbind_s(ldap_connection);
        ldap_connection = 0;
    }

    // Create the LDAP connection
    ldap_connection = ldap_init(dialog_connect.getServer().ascii(), dialog_connect.getPort());
    if (!ldap_connection) {
        log_text = "";
        log
            << "<font color=\"#FF0000\">Unable to connect to Kolab LDAP server "
            << dialog_connect.getServer()
            << dialog_connect.getPort()
            << " ("
            << ldap_err2string(errno)
            << ")</font>";
        txtLog->append(log_text);
        return;
    }

    // Set the protocol version
    if (ldap_set_option(ldap_connection, LDAP_OPT_PROTOCOL_VERSION, &ldap_protocol) != LDAP_OPT_SUCCESS) {
        log_text = "";
        log
            << "<font color=\"#FF0000\">Unable to initiate LDAP protocol version "
            << ldap_protocol
            << "</font>";
        txtLog->append(log_text);
        return;
    }

    QString bind_dn = dialog_connect.getBindDn();
    if (dialog_connect.appendBaseDN())
        bind_dn += ", " + ldap_base_dn;

    // Bind to the specified DN
    if (ldap_simple_bind_s(ldap_connection, bind_dn.ascii(), dialog_connect.getBindPw().ascii()) != LDAP_OPT_SUCCESS) {
        log_text = "";
        log
            << "<font color=\"#FF0000\">Unable to bind to LDAP DN "
            << bind_dn
            << "</font>";
        txtLog->append(log_text);
        return;
    }

    ldap_connected = true;

    log_text = "";
    log
        << "Connected to Kolab LDAP server "
        << dialog_connect.getServer()
        << ":"
        << dialog_connect.getPort();
    txtLog->append(log_text);

    statusBar()->message("Connected");

    slotReloadList();
}

void frmMain::slotReloadList()
{
    // Set up our logging variables
    QString log_text;
    QTextOStream log(&log_text);

    // Clear the entry list
    lstEntries->clear();
    lstAttributes->clear();

    if (!ldap_connected) {
        txtLog->append("Unable to refresh entry list; not connected to a Kolab LDAP server");
        return;
    }

    txtLog->append("Reloading entry list");

    LDAPMessage * ldap_message = 0;

    // Search for all the users
    int result = ldap_search_s(
        ldap_connection,
        ldap_base_dn.ascii(),
        LDAP_SCOPE_ONELEVEL,
        "(mail=*)",
        ATTR_USER_LIST,
        0,
        &ldap_message
    );

    if (result != LDAP_OPT_SUCCESS) {
        log_text = "";
        log
            << "<font color=\"#FF0000\">Unable to search in LDAP DN "
            << ldap_base_dn
            << "</font>";
        txtLog->append(log_text);
        return;
    }

    if (!ldap_count_entries(ldap_connection, ldap_message)) {
        log_text = "";
        log
            << "No user objects found in LDAP DN "
            << ldap_base_dn;
        txtLog->append(log_text);
        return;
    }

    // Iterate through the user list, adding each user to the user list view
    LDAPMessage * ldap_entry = ldap_first_entry(ldap_connection, ldap_message);

    UserListViewItem * item;
    char ** attribute_values;

    do {
        // Sanity check to make sure we actually have a mail attribute
        attribute_values = ldap_get_values(ldap_connection, ldap_entry, "mail");
        if (!ldap_count_values(attribute_values)) continue;

        item = new UserListViewItem(lstEntries, ldap_connection, ldap_entry);

        ldap_value_free(attribute_values);
    } while ((ldap_entry = ldap_next_entry(ldap_connection, ldap_entry)) != 0);

    // At the moment we only add user objects to the entry list. In the future
    // more object types can be added here.
}

void frmMain::slotEntryContextPopup( QListViewItem * list_entry )
{
    QPopupMenu* entry_context = new QPopupMenu(this);
    Q_CHECK_PTR(entry_context);

    entry_context->insertItem("Reload &Entry List", this, SLOT(slotReloadList()));
    if (list_entry) {
        entry_context->insertSeparator();
        entry_context->insertItem("Re&load Entry", this, SLOT(slotReloadLdapEntry()));
        entry_context->insertItem("&Remove Entry...",  this, SLOT(slotRemoveLdapEntry()));
    }
    entry_context->insertSeparator();
    entry_context->insertItem("&Add User...", this, SLOT(slotAddUser()));
    if (list_entry) {
        entry_context->insertItem("Change User &Password...", this, SLOT(slotChangePassword()));
    }

    entry_context->exec(QCursor::pos());
    delete entry_context;
}

void frmMain::slotAddUser()
{
    // Set up our logging variables
    QString log_text;
    QTextOStream log(&log_text);

    // Add a new user
    if (!ldap_connected) {
        txtLog->append("Unable to add user; not connected to a Kolab server");
        return;
    }

    dlgAddUser dialog_add_user(this);
    if (dialog_add_user.exec() == QDialog::Rejected) {
        statusBar()->message("Cancelled");
        return;
    }

    QString cn = dialog_add_user.getFirstName() + " " +
                 dialog_add_user.getSurname();

    QString dn = "cn=" + cn + ", " + ldap_base_dn;
    QString uid = dialog_add_user.getEmailAddress();

    LDAPMessage * ldap_message = 0;

    // Do some checks so as to not add users that already exist
    // DN check
    int result = ldap_search_s(
        ldap_connection,
        dn.ascii(),
        LDAP_SCOPE_ONELEVEL,
        "(mail=*)",
        ATTR_USER_LIST,
        0,
        &ldap_message
    );

    if (result == LDAP_OPT_SUCCESS) {
        log_text = "";
        log
            << "<font color=\"#FF0000\">Unable to add user; DN "
            << dn
            << " already exists</font>";
        txtLog->append(log_text);
        return;
    }

    if (result != LDAP_NO_SUCH_OBJECT) {
        log_text = "";
        log
            << "<font color=\"#FF0000\">Unable to check for existing users corresponding to DN "
            << dn
            << "</font>";
        txtLog->append(log_text);
        return;
    }

    // Mail (UID) check
    result = ldap_search_s(
        ldap_connection,
        ldap_base_dn.ascii(),
        LDAP_SCOPE_ONELEVEL,
        (QString("(mail=" + uid + ")")).ascii(),
        ATTR_USER_LIST,
        0,
        &ldap_message
    );

    if (result != LDAP_OPT_SUCCESS) {
        log_text = "";
        log
            << "<font color=\"#FF0000\">Unable to search for UID "
            << uid
            << "</font>";
        txtLog->append(log_text);
        return;
    }

    if (ldap_count_entries(ldap_connection, ldap_message)) {
        log_text = "";
        log
            << "<font color=\"#FF0000\">Unable to add user; email address "
            << uid
            << " is already is use</font>";
        txtLog->append(log_text);
        return;
    }

    bool has_quota = !dialog_add_user.getQuota().isEmpty();

    LdapAttrVec attrs;
    initLDAPAttributes(attrs);

    addLDAPAttributes(attrs, "objectClass", "inetOrgPerson");
    addLDAPAttributes(attrs, "mail", uid);
    addLDAPAttributes(attrs, "uid", uid);
    addLDAPAttributes(attrs, "sn", dialog_add_user.getSurname());
    addLDAPAttributes(attrs, "cn", cn);
    addLDAPAttributes(attrs, "userPassword", dialog_add_user.getPassword());

    if (has_quota)
        addLDAPAttributes(attrs, "userquota", dialog_add_user.getQuota());

    result = ldap_add_s(ldap_connection, dn.ascii(), &attrs[0]);

    clearLDAPAttributes(attrs);

    if (result != LDAP_SUCCESS) {
        QString error_text = ldap_err2string(result);
        txtLog->append("<font color=\"#FF0000\">" + error_text + "</font>");
        return;
    }

    // Load the new user into the entry list
    if (ldap_search_s(
            ldap_connection,
            dn.ascii(),
            LDAP_SCOPE_BASE,
            "(mail=*)",
            ATTR_USER_LIST,
            0,
            &ldap_message
        ) != LDAP_OPT_SUCCESS)
    {
        log_text = "";
        log
            << "<font color=\"#FF0000\">Unable to search in LDAP DN "
            << dn
            << "</font>";
        txtLog->append(log_text);
        return;
    }

    int count = ldap_count_entries(ldap_connection, ldap_message);
    if (!count) {
        log_text = "";
        log
            << "No user objects found in LDAP DN "
            << dn;
        txtLog->append(log_text);
        return;
    }
    if (count > 1) {
        log_text = "";
        log
            << "More than one object found for LDAP DN "
            << dn;
        txtLog->append(log_text);
        return;
    }

    LDAPMessage * ldap_entry = ldap_first_entry(ldap_connection, ldap_message);

    UserListViewItem * item;
    char ** attribute_values;

    attribute_values = ldap_get_values(ldap_connection, ldap_entry, "mail");
    if (!ldap_count_values(attribute_values)) {
        log_text = "";
        log
            << "No attributes found for LDAP DN "
            << dn;
        txtLog->append(log_text);
        return;
    }

    item = new UserListViewItem(lstEntries, ldap_connection, ldap_entry);

    ldap_value_free(attribute_values);

    txtLog->append("Successfully added entry " + dn);
}

void frmMain::slotRemoveLdapEntry()
{
    // Remove the selected entry (if there is a selected entry)
    QListViewItem * list_entry = lstEntries->selectedItem();
    LdapListViewItem * ldap_item;

#ifdef WIN32
    if (!list_entry) return;
    ldap_item = (LdapListViewItem *)list_entry;
#else
    if (!list_entry || !(ldap_item = dynamic_cast<LdapListViewItem *>(list_entry)))
        return;
#endif

    QString dn = ldap_item->getDN();

    if (ldap_item->awaitingRemoval()) {
        txtLog->append("Entry " + dn + " has already been flagged for deletion");
        return;
    }

    if (QMessageBox::warning(this, "Confirm Removal",
                             "Are you sure you wish to remove the LDAP entry with DN\n"
                             + dn,
                             "&Remove Entry", "&Abort",
                             0, 0, 1) == 1) {
        statusBar()->message("Aborted");
        return;
    }

    QString result = ldap_item->remove();
    if (result != QString::null) {
        txtLog->append("<font color=\"#FF0000\">Unable to remove entry " + dn + ": " + result + "</font>");
        return;
    }

    bool dne = false;
    result = ldap_item->reload(&dne);
    if (dne) {
        lstEntries->takeItem(list_entry);
        return;
    }
    if (result != QString::null) {
        txtLog->append("<font color=\"#FF0000\">Unable to reload entry " + dn + ": " + result + "</font>");
        return;
    }

    lstAttributes->clear();
    ldap_item->populateListView(lstAttributes);

    // Repaint the item to show that is is awaiting removal
    list_entry->repaint();

    txtLog->append("Removed entry " + dn);
}

void frmMain::slotNukeLdapEntry()
{
    // Nuke the selected entry (if there is a selected entry)
    QListViewItem * list_entry = lstEntries->selectedItem();
    LdapListViewItem * ldap_item;

#ifdef WIN32
    if (!list_entry) return;
    ldap_item = (LdapListViewItem *)list_entry;
#else
    if (!list_entry || !(ldap_item = dynamic_cast<LdapListViewItem *>(list_entry)))
        return;
#endif

    QString dn = ldap_item->getDN();

    if (QMessageBox::warning(this, "Confirm Nuke",
                             "Are you sure you wish to nuke the LDAP entry with DN\n"
                             + dn,
                             "&Nuke Entry", "&Abort",
                             0, 0, 1) == 1) {
        statusBar()->message("Aborted");
        return;
    }

    QString result = ldap_item->nuke();
    if (result != QString::null) {
        txtLog->append("<font color=\"#FF0000\">Unable to nuke entry " + dn + ": " + result + "</font>");
        return;
    }

    lstEntries->takeItem(list_entry);

    txtLog->append("Nuked entry " + dn);
}

void frmMain::slotReloadLdapEntry()
{
    // Reload the attributes of the selected entry (if there is a selected entry)
    QListViewItem * list_entry = lstEntries->selectedItem();
    LdapListViewItem * ldap_item;

#ifdef WIN32
    if (!list_entry) return;
    ldap_item = (LdapListViewItem *)list_entry;
#else
    if (!list_entry || !(ldap_item = dynamic_cast<LdapListViewItem *>(list_entry)))
        return;
#endif

    QString dn = ldap_item->getDN();

    txtLog->append("Reloading entry " + dn);

    bool dne = false;
    QString result = ldap_item->reload(&dne);
    if (dne) {
        lstEntries->takeItem(list_entry);
        txtLog->append("Entry " + dn + " no longer exists; removing...");
        return;
    }
    if (result != QString::null) {
        txtLog->append("<font color=\"#FF0000\">Unable to reload entry " + dn + ": " + result + "</font>");
        return;
    }

    lstAttributes->clear();
    ldap_item->populateListView(lstAttributes);
}

void frmMain::slotShowEntry( QListViewItem * list_entry )
{
    // Show the attributes of the selected entry
    LdapListViewItem * ldap_item;

#ifdef WIN32
    if (!list_entry) return;
    ldap_item = (LdapListViewItem *)list_entry;
#else
    if (!list_entry || !(ldap_item = dynamic_cast<LdapListViewItem *>(list_entry)))
        return;
#endif

    QString result = ldap_item->getAttrs();
    if (result != QString::null) {
        txtLog->append("<font color=\"#FF0000\">Unable to show entry: " + result + "</font>");
        return;
    }

    lstAttributes->clear();
    ldap_item->populateListView(lstAttributes);
}

void frmMain::slotChangePassword()
{
    // Attempt to change the password of the selected entry (if there is a selected entry)
    QListViewItem * list_entry = lstEntries->selectedItem();
    UserListViewItem * user_item;

#ifdef WIN32
    if (!list_entry) return;
    user_item = (UserListViewItem *)list_entry;
#else
    if (!list_entry || !(user_item = dynamic_cast<UserListViewItem *>(list_entry)))
        return;
#endif

    QString dn = user_item->getDN();

    if (user_item->awaitingRemoval()) {
        txtLog->append("Entry " + dn + " has been flagged for deletion; ignoring password change request");
        return;
    }

    QString cn = user_item->getAttrValue("cn");
    if (cn == QString::null) {
        txtLog->append("<font color=\"#FF0000\">Error retrieving attribute \"cn\" from DN " + dn + "</font>");
        return;
    }

    dlgPassword dialog_password(this);
    dialog_password.setUser(cn);
    if (dialog_password.exec() == QDialog::Rejected) {
        statusBar()->message("Cancelled");
        return;
    }

    QString result = user_item->changePassword(dialog_password.getPassword());
    if (result != QString::null) {
        txtLog->append("<font color=\"#FF0000\">Unable to change password for user " + cn + ": " + result + "</font>");
        return;
    }

    txtLog->append("Successfully changed password for user " + cn);
}

void frmMain::slotRedrawEntries()
{
#ifdef WIN32
    lstEntries->repaintContents();
#endif
}

void frmMain::slotRedrawAttributes()
{
#ifdef WIN32
    lstAttributes->repaintContents();
#endif
}
